<?php

namespace Teknisa\ACL\Controllers;

use Exception;
use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Teknisa\ACL\Services;
use Doctrine\DBAL\Connection;
use Teknisa\ACL\Util\AclUtilities;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\ACL\Util\InstanceProvider;

class GrupoOperador extends CrudWithParameterBag {

    protected $entityManager;

    protected $dataSourceNameArray = array (
        'grupo_operadores' => 'ger02000_grupoOperador',
        'grupo' => 'grupopcont'
    );

    protected $parameterBagColumnsArray = array (
        'grupo_operadores' =>  array('NRORG', 'CDGRUPOPER','NRPERFILACESSO'),
        'grupo' =>  array('NRPERFILACESSO'),
    );

    public function __construct ($crudService, $parameterBag, Manager $dataSourceManager,\Doctrine\ORM\EntityManager $entityManager) {
        $this->crudService       = $crudService;
        $this->parameterBag      = $parameterBag;
        $this->dataSourceManager = $dataSourceManager;
        $this->entityManager = $entityManager;

    }

    public function findGrupoOperadores(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['grupo_operadores'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['grupo_operadores'];
        parent::find($request, $response);
    }

    public function SaveGrupo ($rows){
        $rows = $rows->getDataSet()->getRows();
        $labels = $rows[0]['ACL_LABELS_TRADUCTION_LOG'];;
        foreach($rows as $row){
            $params = array(
                ":CDGRUPOPER"=> $row['CDGRUPOPER'],
                ":NRORG" => $row['NRORG'],
                ":NRPERFILACESSO" =>$row['NRPERFILACESSO']
            );
            $query = $this->entityManager->getConnection()->executeQuery(self::INSERT_GRUPO, $params);
            $arrayValores = [$labels['label'], $row['CDGRUPOPER'], $row['NRPERFILACESSO'], $row['DSGRUPOPER']];
            $arrayNomes = ['Tela', $labels['CDGRUPOPER'], 'NRPERFILACESSO', $labels['DSGRUPOPER']];
            $descricao = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
            InstanceProvider::getLibLog()->logRegister($row['CDGRUPOPER'].'_'.$row['NRPERFILACESSO'], $descricao, '1', 'GRUPOPCONT', '/Novo_Grupo/save');
        }
    }

     public function findGrupo(DTO\Request\Filter $request, DTO\Response $response) {
      $this->dataSourceName = $this->dataSourceNameArray['grupo'];
      $this->parameterBagColumns = $this->parameterBagColumnsArray['grupo'];
      $conditions = $request->getFilterCriteria()->getConditions();
      unset($conditions[0]);
      $newRequest = new DTO\Request\Filter(
            new FilterCriteria(
                $request->getFilterCriteria()->getDataSourceName(),
                $conditions,
                $request->getFilterCriteria()->getPage(),
                $request->getFilterCriteria()->getPageSize()
            ),
            $request->getMethod(),
            $request->getRoutePath(),
            $request->getUserId()
        );
        parent::find($newRequest, $response);
    }

    public function DeleteGrupo(DTO\Request $request, DTO\Response $response) {
        $rows  = $request->getDataSet()->getRows();
        $labels = $rows[0]['ACL_LABELS_TRADUCTION_LOG'];;
        $arrayValores = [$labels['label'], $rows[0]['CDGRUPOPER'], $rows[0]['NRPERFILACESSO']];
        $arrayNomes = ['Tela', $labels['CDGRUPOPER'], $labels['NRPERFILACESSO']];
        $descricao = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
        InstanceProvider::getLibLog()->logRegister($rows[0]['CDGRUPOPER'].'_'.$rows[0]['NRPERFILACESSO'], $descricao, '3', 'GRUPOPCONT', '/grupopcont/remove');
        $this->dataSourceName = $this->dataSourceNameArray['grupo'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['grupo'];
        parent::delete($request, $response);
    }

    const INSERT_GRUPO = "
        INSERT INTO GRUPOPCONT 
                  (CDGRUPOPER,NRORG,NRPERFILACESSO) 
           VALUES (:CDGRUPOPER, :NRORG, :NRPERFILACESSO)
	";

    public function SaveGrupoper($row)
    {
        $rows = $row->getDataSet()->getRows();
        $labels = $rows[0]['ACL_LABELS_TRADUCTION_LOG'];
       $operatorService = InstanceProvider::getService()->getOperator();

        foreach($rows as $row){
            $gropor[]=$row['CDGRUPOPER'];
            $operator['CDOPERADOR']=$row['CDOPERADOR'];
            $nrperfilacesso['NRPERFILACESSO']=isset($row['NRPERFILACESSO']) ? $row['NRPERFILACESSO'] : null;
            $dsgrupoper = $row['DSGRUPOPER'];
        }
        $operatorService->SaveGrupoper( $gropor,$operator,$nrperfilacesso, $labels, $dsgrupoper);
    }

    public function DeleteGrupoper(DTO\Request $request, DTO\Response $response) {
        $rows  = $request->getDataSet()->getRows();
        $labels = $rows[0]['ACL_LABELS_TRADUCTION_LOG'];
        $arrayValores = [$labels['label'], $rows[0]['CDGRUPOPER'], $rows[0]['CDOPERADOR']];
        $arrayNomes = ['Tela', $labels['CDGRUPOPER'], 'CDOPERADOR'];
        $descricao = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
        InstanceProvider::getLibLog()->logRegister($rows[0]['CDGRUPOPER'].'_'.$rows[0]['CDOPERADOR'], $descricao, '3', 'OPERGRUPOP', '/opergrupop/remove');
        $this->dataSourceName = 'grupoper';
        $this->parameterBagColumns = 'CDOPERADOR';
        parent::delete($request, $response);
    }

    /**
     * @param DTO\Request\Filter $request
     * @param DTO\Response $response
     * @throws Exception
     */
    public function findOpergrupop(DTO\Request\Filter $request, DTO\Response $response)
    {
        $conditions = $request->getFilterCriteria()->getConditions();
        $cdoperador = AclUtilities::getValueInFilterDataSourceByName($conditions, 'CDOPERADOR', true);
        $conditions = InstanceProvider::getService()->getGrupoOperador()->findOpergrupop($conditions, $cdoperador);
        $response->addDataSet(new DataSet("findOpergrupop", $conditions));
    }

    /**
     * @param DTO\Request\DataSet $request
     * @param DTO\Response $response
     */
    public function updateOpergrupop(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $labels = $row['ACL_LABELS_TRADUCTION_LOG'];
            $idmenuprinop = $row['IDMENUPRINOP'];
            $cdgrupoper = $row['CDGRUPOPER'];
            $cdoperador = $row['CDOPERADOR'];
            $dsgrupoper = $row['DSGRUPOPER'];
            InstanceProvider::getService()->getGrupoOperador()->updateOpergrupop($idmenuprinop, $cdgrupoper, $cdoperador,  $labels, $dsgrupoper);
            $response->addNotification(new DTO\Response\Notification("Registro atualizado com sucesso.", DTO\Response\Notification::TYPE_SUCCESS));
        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param DTO\Request\Filter $request
     * @param DTO\Response $response
     * @throws Exception
     */
    public function findGrupooperByGrupopcont(DTO\Request\Filter $request, DTO\Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $cdoperador = AclUtilities::getValueInFilterDataSourceByName($conditions, 'CDOPERADOR', true);
            $nrperfilacesso = AclUtilities::getValueInFilterDataSourceByName($conditions, 'NRPERFILACESSO', true);
            $conditions = InstanceProvider::getService()->getGrupoOperador()->findGrupooperByGrupopcont($cdoperador, $nrperfilacesso);
            $response->addDataSet(new DataSet("findGrupooperByGrupopcont", $conditions));
        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }
}
