<?php

namespace Teknisa\ACL\Services;

use Teknisa\ACL\Repositories\AccessProfile as AccessProfileRepository;
use Teknisa\ACL\Repositories\PerfAcessParc;
use Teknisa\ACL\Repositories\ZhuAccessControlRule;
use Teknisa\Libs\Util\Environment;
use Doctrine\ORM\EntityManager;
use Teknisa\ACL\Util\Utilities;
use Teknisa\ACL\Util\InstanceProvider;
use Zeedhi\ZhuAcl\Util\TableProperties;
use Teknisa\ACL\Util\AclUtilities;
use Doctrine\DBAL\Connection;

class AccessProfile{

    const MAXNRPERFILACESSO = "MAXNRPERFILACESSO";
    const NRORG = "NRORG";
    const NRUSERGROUP = "NRUSERGROUP";
    const NRPERFACESSOPAI = "NRPERFACESSOPAI";
    const NRORGINCLUSAO = "NRORGINCLUSAO";
    const NRORGULTATU = "NRORGULTATU";
    const DTINCLUSAO = "DTINCLUSAO";
    const DTULTATU = "DTULTATU";
    const CDOPERINCLUSAO = "CDOPERINCLUSAO";
    const CDOPERULTATU = "CDOPERULTATU";
    const NRPERFILACESSO = "NRPERFILACESSO";
    const NMOPERINCLUSAO = "NMOPERINCLUSAO";
    const NMOPERULTATU = "NMOPERULTATU";

    /** @var AccessProfileRepository */
    protected $accessProfileRepository;

    /** @var PerfAcessParc */
    protected $perfAcessParcRepository;

    /** @var ZhuAccessControlRule */
    protected $accessControlRuleRepository;

    /** @var Environment */
    protected $environment;

    public function __construct(AccessProfileRepository $accessProfileRepository, PerfAcessParc $perfAcessParcRepository,
                                ZhuAccessControlRule $accessControlRuleRepository, Environment $environment) {
        $this->accessProfileRepository = $accessProfileRepository;
        $this->perfAcessParcRepository = $perfAcessParcRepository;
        $this->accessControlRuleRepository = $accessControlRuleRepository;
        $this->environment = $environment;
    }

    public function getMaxNrPerfilAcesso() {
        return InstanceProvider::getNewCode()->getCodeByTableName('PERFILACESSO', $this->environment->getCurrentOrganizationId());
    }

    public function setDefaultValues(&$row) {
        $this->setNrOrgValues($row);
        $this->setDateValues($row);
        $this->setOperatorValues($row);

        $row[self::NRUSERGROUP] = null;
        $row[self::NRPERFACESSOPAI] = null;
    }

    protected function setNrOrgValues(&$row) {
        $nrOrg = $this->getNrOrgFilter();
        $row[self::NRORG] = $this->setValueIfDontExist($row, self::NRORG, $nrOrg);
        $row[self::NRORGINCLUSAO] = $this->setValueIfDontExist($row, self::NRORGINCLUSAO, $nrOrg);
        $row[self::NRORGULTATU] = $nrOrg;
    }

    protected function setDateValues(&$row) {
        $time = new \DateTime();
        $timeFormatted = $time->format('d/m/Y H:i:s');
        $row[self::DTINCLUSAO] = $this->setValueIfDontExist($row, self::DTINCLUSAO, $timeFormatted);
        $row[self::DTULTATU] = $timeFormatted;
    }

    protected function setOperatorValues(&$row) {
        $cdOperador = $this->environment->getCurrentUserId();
        $row[self::CDOPERINCLUSAO] = $this->setValueIfDontExist($row, self::CDOPERINCLUSAO, $cdOperador);
        $row[self::CDOPERULTATU] = $cdOperador;
    }

    public function getCurrentUserName() {
        return $this->environment->getCurrentUserName();
    }

    public function deleteDependencies($profilesToDelete, $labels, $nmperfilacess) {
        $params = array (
            self::NRORG =>  $this->getNrOrgFilter(),
            self::NRPERFILACESSO =>  $profilesToDelete
        );
        $typeBindParams = array(
            self::NRPERFILACESSO =>  Connection::PARAM_INT_ARRAY
        );
        $this->accessControlRuleRepository->deleteAssociatedRules($params, $typeBindParams);
        $this->perfAcessParcRepository->deleteAssociatedPerfAcessParc( $params, $typeBindParams);
        $this->accessProfileRepository->deleteAssociatedPerfAcessFili($profilesToDelete, $params);
        $arrayValores = [$labels['label'], $nmperfilacess, $profilesToDelete[0]];
        $arrayNomes = ['Tela', $labels['NMPERFILACESSO'], 'NRPERFILACESSO'];
        $descricao = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
        InstanceProvider::getLibLog()->logRegister($profilesToDelete[0], $descricao, '3', 'PERFILACESSO', '/accessProfile/remove');
    }

    public function getNrOrgFilter() {
        return $this->environment->getCurrentOrganizationId();
    }

    protected function setValueIfDontExist($row, $property, $value) {
        return !empty($row[$property]) ? $row[$property] : $value;
    }
    
    public function findAccessControllUtilization(){
        $nrorg = InstanceProvider::getEnvironment()->getOrganizationId();
        return $this->accessProfileRepository->findAccessControllUtilization($nrorg);
    }

    public function validateActivation(){
        $nrorg = InstanceProvider::getEnvironment()->getOrganizationId();
        return $this->accessProfileRepository->validateActivation($nrorg);
    }
    
    public function changeAccessProfile($idutctrlacesweb, $labels){
        $idutctrlaceswebAux = $idutctrlacesweb;
        $nrorg = InstanceProvider::getEnvironment()->getOrganizationId();
        $findAccessControllUtilization = $this->findAccessControllUtilization($nrorg);
        $idaces = $idutctrlacesweb == 'N'?'S':'N';
        if(!empty($findAccessControllUtilization)) {    
            $acessProfileOld = $this->accessProfileRepository->findAccessProfileOld($nrorg);
            $arrayValores = [$labels['label']];
            $arrayNomes = ['Tela'];
            if ($idutctrlaceswebAux !== $idaces) {
                array_push($arrayValores, $idaces);
                array_push($arrayNomes, 'IDUTCTRLACESWEB');
            }
            $descricao = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
            InstanceProvider::getLibLog()->logRegister($acessProfileOld[0]['CDPARGERAL'], $descricao, '2', 'PARAMGERAL', '/changeAccessProfile/save');
            $this->accessProfileRepository->changeAccessProfile($nrorg, $idaces);
        } else {
            $interval = 1;
            $colSize = 4;
            $cdpargeral = InstanceProvider::getNewCode()->getCodeByTableName('PARAMGERAL', $nrorg, $interval, $colSize);
            $this->accessProfileRepository->insertAccessProfile($nrorg, $idaces, $cdpargeral);
            $arrayValores = [$labels['label'], $idaces];
            $arrayNomes = ['Tela', 'IDUTCTRLACESWEB'];
            $descricao = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
            InstanceProvider::getLibLog()->logRegister($cdpargeral, $descricao, '1', 'PARAMGERAL', '/changeAccessProfile/save');
        }
        return array('idutctrlacesweb' => $idaces);
    }
    
    public function findPerfAcessControl($nrAccessProfile){
        $nrorg = InstanceProvider::getEnvironment()->getOrganizationId();
        return $this->accessProfileRepository->findPerfAcessControl($nrorg, $nrAccessProfile);
    }
    
    public function findPerfAccessByControlProfile($nrAccessProfile){
        $nrorg = InstanceProvider::getEnvironment()->getOrganizationId();
        return $this->accessProfileRepository->findPerfAccessByControlProfile($nrorg, $nrAccessProfile);
    }
    
    public function savePerfAccessByControlProfile($rows, $labels){
        $entityManager = InstanceProvider::getEntityManager();
        $connection = $entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $libEnvironment = InstanceProvider::getLibEnvironment();
            $organizationId = $libEnvironment->getCurrentOrganizationId();
            foreach($rows as $accessProfile){
                $accessProfile['NRORG']             = $organizationId;
                $accessProfile['NRORGINCLUSAO']     = $organizationId;
                $accessProfile['CDOPERINCLUSAO']    = $libEnvironment->getCurrentUserId();
                $this->accessProfileRepository->saveAccessProfileByControlProfile($accessProfile);
                $arrayValores = ['operator', $accessProfile['NRPERFACESSCONT'], $accessProfile['NRPERFILACESSO']];
                $arrayNomes = ['Tela', 'NRPERFACESSCONT', $labels['NMPERFILACESSO']];
                $descricao = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
                InstanceProvider::getLibLog()->logRegister($accessProfile['NRPERFACESSCONT'].'_'.$accessProfile['NRPERFILACESSO'], $descricao, '1', 'PERFACESSCONTROL', '/perfAccessByControlProfile/save');
            }
            $entityManager->flush();
            $connection->commit();
            return (array());
        } catch (\Exception $e){
            $connection->rollBack();
            throw $e;
        }
    }
    
    public function deleteControlProfile($nrAccesProfile, $nrControlProfile, $labels)
    {
        $organizationId = InstanceProvider::getLibEnvironment()->getCurrentOrganizationId();
        $this->accessProfileRepository->deleteControlProfile($nrAccesProfile, $nrControlProfile, $organizationId);
        $arrayValores = [$labels['label'], $nrControlProfile, $nrAccesProfile];
        $arrayNomes = ['Tela', 'NRPERFACESSCONT', $labels['NMPERFILACESSO']];
        $descricao = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
        InstanceProvider::getLibLog()->logRegister(
            $nrControlProfile . '_' . $nrAccesProfile,
            $descricao,
            '3',
            'PERFACESSCONTROL',
            '/perfAcessControl/remove'
        );
    }
    
    public function changeAccessControl($idutlperfilfilial, $labels){
        $nrorg = InstanceProvider::getEnvironment()->getOrganizationId();
        $acessProfileOld = $this->accessProfileRepository->findAccessProfileOld($nrorg);
        if ($acessProfileOld[0]['IDUTLPERFILFILIAL'] !== $idutlperfilfilial)   {
            $arrayValores = [$labels['label'], $idutlperfilfilial];
            $arrayNomes = ['Tela', 'IDUTLPERFILFILIAL'];
            $descricao = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
            InstanceProvider::getLibLog()->logRegister($acessProfileOld[0]['CDPARGERAL'], $descricao, '2', 'PARAMGERAL', '/changeAccessControl/save');
        }
        $this->accessProfileRepository->changeAccessControl($nrorg, $idutlperfilfilial);
        return array('idutlperfilfilial' => $idutlperfilfilial);
    }

    public function saveLogLib($rows, $labels) {
        $nrperfilacesso = $rows[0]['NRPERFILACESSO'];
        $idPerfilAcessoAdm = $rows[0]['IDPERFILACESSOADM'];
        $idativo = $rows[0]['IDATIVO'];
        $nmPerfAcess = $rows[0]['NMPERFILACESSO'];
        $idBlockFilial = isset($rows[0]['IDBLOCKFILIAL']) ? $rows[0]['IDBLOCKFILIAL'] : 'N';
        if ($rows[0]['__is_new'] === true) {
            $arrayValores = [$labels['label'], $nrperfilacesso, $idPerfilAcessoAdm, $idativo, $nmPerfAcess, $idBlockFilial];
            $arrayNomes = ['Tela', 'NRPERFILACESSO', $labels['IDPERFILACESSOADM'], $labels['IDATIVO'], $labels['NMPERFILACESSO'], $labels['IDBLOCKFILIAL']];
            $descricao = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
            InstanceProvider::getLibLog()->logRegister(
                $nrperfilacesso,
                $descricao,
                '1',
                'PERFILACESSO',
                '/accessProfile/save'
            );
        } else {
            $perfAcessoOld = $this->accessProfileRepository->findValAntigoPerfAcess($nrperfilacesso);
            $arrayValores = [$labels['label']];
            $arrayNomes = ['Tela'];
            if ($perfAcessoOld[0]['IDPERFILACESSOADM'] !== $idPerfilAcessoAdm) {
                array_push($arrayValores, $idPerfilAcessoAdm);
                array_push($arrayNomes, $labels['IDPERFILACESSOADM']);
            }
            if ($perfAcessoOld[0]['IDATIVO'] !== $idativo) {
                array_push($arrayValores, $idativo);
                array_push($arrayNomes, $labels['IDATIVO']);
            }
            if ($perfAcessoOld[0]['NMPERFILACESSO'] !== $nmPerfAcess) {
                array_push($arrayValores, $nmPerfAcess);
                array_push($arrayNomes, $labels['NMPERFILACESSO']);
            }
            if ($perfAcessoOld[0]['IDBLOCKFILIAL'] !== $idBlockFilial) {
                array_push($arrayValores, $idBlockFilial);
                array_push($arrayNomes, $labels['IDBLOCKFILIAL']);
            }
            $descricao = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
            InstanceProvider::getLibLog()->logRegister($nrperfilacesso, $descricao, '2', 'PERFILACESSO', '/accessProfile/save');
        }
    }

    public function deleteLogLibExportRules($row, $labels) {
        $nrorg = InstanceProvider::getEnvironment()->getOrganizationId();
        foreach ($row['profilesTo'] as $profilesTo) {
            $allRules = $this->accessProfileRepository->findRules($nrorg, $profilesTo);
            if ($allRules) {
                foreach ($allRules as $ruleTo) {
                    $arrayValores = [$labels['label'], $ruleTo['ID'], $ruleTo['GROUP_ID'], $ruleTo['RULE']];
                    $arrayNomes = ['Tela', 'ID', 'GROUP_ID', 'RULE'];
                    $descricao = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
                    InstanceProvider::getLibLog()->logRegister(
                        $ruleTo['ID'],
                        $descricao,
                        '3',
                        'ZHU_ACCESS_CONTROL_RULE',
                        '/acl_zhu_acl_element_exportRules/save'
                    );
                }
            }
        }
    }

    public function insertLogLibExportRules($row, $labels) {
        $nrorg = InstanceProvider::getEnvironment()->getOrganizationId();
        $rules = $this->accessProfileRepository->findRules($nrorg, $row['profileFrom']);
        if ($rules) {
            foreach($rules as $ruleFrom){
                foreach ($row['profilesTo'] as $profilesTo) {
                    $allRules = $this->accessProfileRepository->findRules($nrorg, $profilesTo);
                    foreach ($allRules as $ruleTo) {
                        $arrayValores = [$labels['label'], $ruleFrom['ID'], $ruleFrom['GROUP_ID'], $ruleFrom['RULE']];
                        $arrayNomes = ['Tela', 'ID', 'GROUP_ID', 'RULE'];
                        $descricao = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
                        InstanceProvider::getLibLog()->logRegister(
                            $ruleTo['ID'],
                            $descricao,
                            '1',
                            'ZHU_ACCESS_CONTROL_RULE',
                            '/acl_zhu_acl_element_exportRules/save'
                        );
                    }
                }
            }
        }
    }

    public function logSaveParentMenuContainerRule($row, $labels) {
        $element = $this->accessProfileRepository->findElement($row['UNIQUE_ID'], $row['PROJECT_ID']);
        $arrayValores = [$labels['label'], $element[0]['TYPE'], $element[0]['UNIQUE_ID'], $element[0]['PROJECT_ID'], $element[0]['NAME'], $element[0]['LABEL']];
        $arrayNomes = ['Tela', 'TYPE', 'UNIQUE_ID', 'PROJECT_ID', 'NAME', 'LABEL'];
        $descricaoElement = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
        InstanceProvider::getLibLog()->logRegister(
            $element[0]['ID'],
            $descricaoElement,
            '1',
            'ZHU_ACCESS_CONTROL_ELEMENT',
            '/acl_zhu_acl_element_rules/save'
        );
        $rule = $this->accessProfileRepository->findRuleElement($row['ORGANIZATION_ID'], $row['GROUP_ID'], $element[0]['ID']);
        $arrayValores = [$labels['label'], $rule[0]['ORGANIZATION_ID'], $rule[0]['ACCESS_CONTROL_ELEMENT_ID'], $rule[0]['NAME'], $rule[0]['GROUP_ID'], $rule[0]['RULE']];
        $arrayNomes = ['Tela', 'ORGANIZATION_ID', 'ACCESS_CONTROL_ELEMENT_ID', 'NAME', 'GROUP_ID', 'RULE'];
        $descricaoRule = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
        InstanceProvider::getLibLog()->logRegister(
            $rule[0]['ID'],
            $descricaoRule,
            '1',
            'ZHU_ACCESS_CONTROL_RULE',
            '/acl_zhu_acl_element_rules/save'
        );
    }

    public function logSaveRule($row, $labels) {
        $element = $this->accessProfileRepository->findElement($row['UNIQUE_ID'], $row['PROJECT_ID']);
        $arrayValores = [$labels['label'], $element[0]['TYPE'], $element[0]['UNIQUE_ID'], $element[0]['PROJECT_ID'], $element[0]['NAME'], $element[0]['LABEL']];
        $arrayNomes = ['Tela', 'TYPE', 'UNIQUE_ID', 'PROJECT_ID', 'NAME', 'LABEL'];
        $descricaoElement = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
        InstanceProvider::getLibLog()->logRegister(
            $element[0]['ID'],
            $descricaoElement,
            '1',
            'ZHU_ACCESS_CONTROL_ELEMENT',
            '/acl_zhu_acl_element_rules/save'
        );
        $rule = $this->accessProfileRepository->findRuleElement($row['ORGANIZATION_ID'], $row['GROUP_ID'], $element[0]['ID']);
        $arrayValores = [$labels['label'], $rule[0]['ORGANIZATION_ID'], $rule[0]['ACCESS_CONTROL_ELEMENT_ID'], $rule[0]['NAME'], $rule[0]['GROUP_ID'], $rule[0]['RULE']];
        $arrayNomes = ['Tela', 'ORGANIZATION_ID', 'ACCESS_CONTROL_ELEMENT_ID', 'NAME', 'GROUP_ID', 'RULE'];
        $descricaoRule = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
        InstanceProvider::getLibLog()->logRegister(
            $rule[0]['ID'],
            $descricaoRule,
            '1',
            'ZHU_ACCESS_CONTROL_RULE',
            '/acl_zhu_acl_element_rules/save'
        );
    }

    public function logDeleteRule($row, $labels) {
        if(isset($row[TableProperties::COLUMN_RULE_ID])) {
            $rule = $this->accessProfileRepository->findElementRule($row['RULE_ID']);
            $arrayValores = [$labels['label'], $rule[0]['ID']];
            $arrayNomes = ['Tela', 'ID'];
            $descricaoRule = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
            InstanceProvider::getLibLog()->logRegister(
                $rule[0]['ID'],
                $descricaoRule,
                '1',
                'ZHU_ACCESS_CONTROL_RULE',
                '/acl_zhu_acl_element_rules/save'
            );
        }
    }

    public function aclZhuAclElementRules($request, $response, $labels) {
        $updatedRules = array();
        $deletedRules = array();

        foreach($request->getDataSet()->getRows() as $row) {
            $uniqueId = $row[TableProperties::COLUMN_UNIQUE_ID];
            $hasContainerRule = isset($row[TableProperties::COLUMN_CONTAINER_ID]) && $row[TableProperties::COLUMN_TYPE] == TableProperties::TYPE_MENU;

            if(isset($row[TableProperties::COLUMN_PARENT_MENU_ID])){
                InstanceProvider::getAcessControlRule()->saveParentMenuRule($row);
                $this->logSaveParentMenuContainerRule($row,$labels);
            }

            if(isset($row['__isDeleted']) && $row['__isDeleted']){
                $this->logDeleteRule($row, $labels);
                $ruleId = InstanceProvider::getAcessControlRuleService()->deleteRule($row);
                if($ruleId){
                    array_push($deletedRules, $ruleId);
                }

                if($hasContainerRule && isset($row['RULE_SAVE_CONTAINER'])){
                    $row[TableProperties::COLUMN_RULE] = $row['RULE_SAVE_CONTAINER'];
                    $containerRule = InstanceProvider::getAcessControlRule()->saveContainerRule($row);
                    $this->logSaveParentMenuContainerRule($row, $labels);
                    $updatedRules[$uniqueId] = array(
                        "menuId" => $ruleId,
                        "containerId" => $containerRule->getId()
                    );
                }
            } else {
                $rule = InstanceProvider::getAcessControlRuleService()->insertRule($row);
                $this->logSaveRule($row, $labels);
                $ruleId = $rule->getId();

                if($hasContainerRule){
                    $containerRule = InstanceProvider::getAcessControlRule()->saveContainerRule($row);
                    $this->logSaveParentMenuContainerRule($row, $labels);
                    $updatedRules[$uniqueId] = array(
                        "menuId" => $ruleId,
                        "containerId" => $containerRule->getId()
                    );
                } else {
                    $updatedRules[$uniqueId] = $ruleId;
                }
            }
        }

        $modifiedRules = array("deleted"=> $deletedRules, "updated"=> $updatedRules);

        return $modifiedRules;
    }
}